// Unité DefType
// Cordier Yves le 27/10/2018

/* 
 Nous allons ici faire un rapide résumé du passage des variables 
 en comparant C++ et java
 ********************************************
 en C C++ ou en java :
 
 int add (int a, int b)
 {
	 a=a+1;
	 return a+b;
 }
 
 ...
 int c=1;
 int d=2;
 int e=add(c,d)
 // au retour e=4 (1+1 + 2) c=1 et d=2
 
 on dit ici que les variables c et d sont passées par valeurs
 i.e. on a recopié la valeur contenu dans la variable c et d pour les 
 affecter de manière locale à a et b dans la fonction
 *********************************************
 *********************************************
 en C++ :
 
 int add (int &a, int b)
 {
	 a=a+1;
	 return a+b;
 }
 
 ...
 int c=1;
 int d=2;
 int e=add(c,d)
 // au retour e=4 (1+1 + 2) c=2 et d=2
 
 on dit ici que les variables c et d sont passées par adresse (ou reférence)
 i.e. on a recopié la valeur de l'adresse des c et d pour de manière locale 
 affecter les variables a et b à la même adresse dans la fonction. Une modification de 
 a entraine donc une modification de char
 
 Or en java ceci est impossible. Pour des variables simples (type de base integer, float, boolean, string )
 java ne passe que des valeurs dans les parametres de fonctions. 
 Mais il faut se rappeler qu'un objet est en fait connu de java 
 comme une adresse. Donc passer un objet revient à passer une adresse et donc à 
 effectuer un passage par référence
 
 Notre fonction s'écrirait donc en java en utilisant les types définis dans
 cette unité
 
 int add(Int a, Int b)
 {
	 a.put(a.get()+1);
	 return a.get()+b.get();
 }
 c'est cete plus "verbeux" mais presque plus clair.
 
 
 Int c = Int(1);
 Int d = Int(2);
 Int x = Int(1);
 int e=add(c,d); // on remarque ici que e est une valeur de type int et non un objet
 // au retour e=4 (1+1 + 2) c.get() renvoi 2 et d.get() renvoi 2
 Int f;
 f.put(add(x,d));
  // au retour f.get() renvoi 4 x.get() renvoi 2 et d.get() renvoi 2
 
 
 Ce que nous avons écrit, peut se généraliser sur les divers types simples

 
 A l'image de String vs string, une class permettant
 de passer des valeurs int par référence dans les procedures
 Int a; // a est un objet contenant un entier sur 32 bits
 
 de passer des valeurs char par référence dans les procedures
 Char a; // a est un objet contenant un boolean un boolean
 
 de passer des valeurs double par référence dans les procedures
 Double a; // a est un objet contenant un double un reel sur 64 bits
 
 de passer des valeurs float par référence dans les procedures
 Float a; // a est un objet contenant un float un reel sur 32 bits
 
 de passer des valeurs boolean par référence dans les procedures
 Boolean a; // a est un objet contenant un boolean un boolean
 */

// ********************************************************

class Int
{
  private int val=0;
  public Int(int v)
  {
    val=v;
  }
  public int get()
  {
    return val;
  }
  public void put(int v)
  {
    val=v;
  }
}
// ********************************************************
class Double
{
  private double val=0;
  public Double(double v)
  {
    val=v;
  }
  public double get()
  {
    return val;
  }
  public void put(double v)
  {
    val=v;
  }
}

// ********************************************************

class Float
{
  private float val=0;
  public Float(float v)
  {
    val=v;
  }
  public float get()
  {
    return val;
  }
  public void put(float v)
  {
    val=v;
  }
}

// ********************************************************

class Boolean
{
  private boolean val=true;
  public Boolean(boolean v)
  {
    val=v;
  }
  public boolean get()
  {
    return val;
  }
  public void put(boolean v)
  {
    val=v;
  }
}
// ********************************************************

class Char
{
  private char val=' ';
  public Char (char v)
  {
    val=v;
  }
  public char get()
  {
    return val;
  }
  public void put(char v)
  {
    val=v;
  }
}
