
// **************************************************************************
// Petit programme d'initiation de creation dune IHM
// sous environnement ProcessingEnvironment
// Le 17/10/2018 Cordier Yves
//
// Vérifié avec les versions minimales 2.2.1 de processing (jusqu'a la 3.4)

/* Architecture des appels de procedures définies par le programmeur
 
 Appelées par interruption souris
 
 void mousePressed() // on a appuyé sur la touche
 void mouseReleased() // on a relaché la touche
 
 Initialisation
 
 void setup()
 
 Procedure principale
 
 draw -------+---- premierTravail
 +---- deuxiemeTravail
 			+---- troixiemeTravail --+-- dessineMire
 			+---- quatriemeTravail --+
 			
 */

/* Nota :
 pour fonctionner ce programme necessite la présence de :
 	- du fichier de definition de la classe Boitetexte (DefBoitetexte.pde)
 	- du fichier de definition de la classe couleur (DefCouleur.pde)
 	dans le repertoire du programme principal
 	
 	- de la definition de la police monotype "DejaVu....vlw" dans le sous répertoire data
 	- du fichier image utilisé "monimage_a_afficher.jpg" dans le sous repertoire DATA
 	
 */

// petit programme d'initiation qui ressemble à un IHM

// un objet couleur pour avoir acces aux entités de la calsse couleur
Couleur coul = new Couleur();
// Les couleurs sont définies par commodité à travers une classe couleur que l'on a instancié via l'objet coul
// cette classe est définie dans le fichier DefCouleur qui doit se trouver dans le même répertoire que le fichier du "programme principal" 

// ================================================
// Unite de gestion des series
// il est indispensable d'importer cette unité pour avoir la gestion des sorties série du PC
// cette unité fait partie de la distribution standard processing

import processing.serial.*;


// une boite texte de dialogue
// positionnée à x=10, y=20
// point de reference en haut à gauche (00)
// 80 caractères de large , 24 lignes 
// taille de caractere 12 
// Tracé d'un cadre (true)
// Couleur du fond, couleur des lettres, couleur du cadre
Boitetext bt1=new Boitetext(10, 20, 00, 80, 10, 12, true, coul.noir, coul.blanc, coul.rouge);

// une autre boite texte  pour envoyer les coordonnées souris
Boitetext bt2=new Boitetext(10, 250, 00, 80, 3, 12, true, coul.noir, coul.blanc, coul.rouge);

// La classe boiitetext est définie dans le fichier DefBoitetexte et donc ce fichier doit être dans le 
// même répertoire que le fichier du "programme principal"



// Un tableau d'objets de type chaine de caractère pour stoker le nom des entrées Sorties Séries du PC
String[] listeSeries; 

// Instanciation d'un objet de type Serial 
Serial portSerie;

// dans ce programme, nous allons utiliser une image
PImage monImage; // on cree un objet image

// =========================================================================
// Le Setup général
void setup()
{
  // definition de la taille fenetre graphique globale de l'application
  size(800, 800);
  // dessin de la boite de dialogue avec son contenu 
  bt1.dessineBoite(true);
  // defini le mode de scrolling ici enroullement
  bt1.setModeEnroule(true);

  // dessin de la boite de dialogue N°2 avec son contenu 
  bt2.dessineBoite(true);
  // defini le mode de scrolling ici scrolling normal
  bt2.setModeEnroule(false);

  // Instanciation d'un objet fonte de caractère pour pouvoir charger la fonte
  PFont mono;
  // chargement de la fonte de caractere à partir du sous répertoire data du programme principal
  // il sagit d'une fonte non proportionelle indispensable au fonctionnement correct en mode TTY de la boite texte
  mono = loadFont("DejaVuSansMono-48.vlw");
  //mono = loadFont("LucidaConsole-48.vlw");
  // mise en place de la fonte pour le tracé graphique des caractères
  textFont(mono);

  // Analyse des ports séries disponibles sur le PC et remplissage du tableau des noms
  listeSeries=Serial.list();
  // affichage des noms sur la boite texte
  bt1.graphprintln("indiquez le N° de la série");
  // Un print un peu particulier imprime toute les cases du tableau en indiquant leur ordre 
  // en incrémentant le caractère indiqué ici 'A'
  bt1.graphprintArray(listeSeries, 'A');
  // un petit baratin pour guider l'utilisateur
  bt1.graphprint("votre choix (A,B,..) ? :");
  // affichage du curseur
  bt1.setCurseur(true);

  // gestion de l'image 
  monImage=loadImage("monimage_a_afficher.jpg"); // charge l'objet monImage avec les données du fichier qui doit se trouver dans
  // le répertoire data
  // on dessine l'image c'est à dire que on decode les données et que l'on dessine pixel par pixel
  // sur la fenètre graphique
  image(monImage, 10, 350); // on l'installe à partir de la position x,y 10,350 point en haut à gauche
}

boolean couleurOn = false; // une variable booleeenne à false

void dessineMire()
{
  if (couleurOn) fill(coul.vert);
  else fill(coul.rouge);
  stroke(coul.blanc); // la couleur du contour
  ellipse(width/2, height/2, 40, 40); // trace de l'ellipse qui est en fait un cercle
}

// ***********************************************************************
String serieSelectionnee =""; // le nom de la sortie série sélectionnée

boolean premierTravail()
{ 
  if (keyPressed) // si une touche apppuyée
  {
    int touche= int(Character.toUpperCase(key)); // on se met en majuscule par défaut
    // touche est du type integer signé (32 bits en java) ou l'on va stocker le code ascii de la touche appuyée
    int reponse=touche-65; // on se souvient que le code ascii de 'A' est 65 => 'A' donnera 0

    if ((reponse>=0) & (reponse<listeSeries.length)) // est on dans les valeurs possible ?
    {
      serieSelectionnee=listeSeries[reponse];
      bt1.graphprintln();
      bt1.graphprintln("Port Série : "+serieSelectionnee);
      // ouverture du port série
      portSerie=new Serial(this, serieSelectionnee, 9600); // this correspond ici à notre application qui est en fait un objet
      // on est obligée de le spécifier car les entrées sortie séries doivent dialoguer avec notre programme
      // n'oublions pas que nous sommes avec un systeme d'exploitation multitache Windows ou unix car android est en fait une surcouche à unix
      return true;
    }
  }
  return false;
}

// ***********************************************************************
boolean deuxiemeTravail(int tempsSortie)
{

  if (millis()>tempsSortie) // on a écoulé les 4s  
  {
    etat=2; // on avanance d'un etat
    bt1.clear(); // on efface l'écran
    int c;
    while (portSerie.read ()>0) {
    }; // on vide le port série à voir i.e tant qu'il y a des caractères à lire
    return true; // on sort victorieux
  } else
  {
    bt1.graphprint((millis()-(tempsSortie-4000))/1000.); // on affiche le temps ecoulé
    bt1.graphprintln(" s ");
  }
  return false; // on sort sans panache
}
// ***********************************************************************
// ici il sagit d'une procedure et non d'une fonction => à la fin de la procedure, on ne renvoi aucune valeur
// en fait
void troixiemeTravail()
{
  println("trois");
  bt1.clear(); // effacer la zone texte
  dessineMire(); // en fait le travail Graphique
}
// ***********************************************************************
int envoiCaractere=0; // Le nombre de caracteres à envoyer via la voie série
void quatriemeTravail()
{
  if (envoiCaractere>0) // on a cliqué plusieurs fois sur la mire
  {
    if (couleurOn) // on est en mire rouge
    {
      portSerie.write('A'); // on envoi sur le port série sélectionné le caractère A ie code 65 en binaire
      bt1.graphprintln("Touché on envoi envoyé 'A'");
    } else
      // on est en mire verte
    {
      portSerie.write('B'); // on envoi sur le port série sélectionné le caractère B ie code 66 en binaire
      bt1.graphprintln("Touché on envoi envoyé 'B'");
    }
    envoiCaractere--; // on decompte les caractères à envoyer
    couleurOn=!couleurOn; // not couleurOn
    dessineMire(); // redessine la mire avec le changement de couleur
  }
}
// =================================================================================================
// Procedure principale
// =================================================================================================
// Le systeme fonctionne sous la forme d'un automate d'état fini ou chaque état est indiqué par son numéro
int etat=0; // la variable de l'automate d'état implementé dans draw
// dans notre cas, nous dénombrerons 4 états
// 0 : l'initialisation qui va prendre la réponse de l'utilisateur sur la sortie série à afficher
// 1 : attendre 4s pour renseigner l'utilisateur du port série sélectionné
// 2 : l'affichage de l'image de fond
// 3 : gzestion de la souris et de la zone graphique cliquée
// 
// Nota : Il est indispensable de ne pas rester dans la procédure draw car le rafraichissement de l'écran du PC ne se fait qu'a la sortie 
// de cette procédure
// typiquemant le programme principal ressemble à
/* void main()
 {
 setup();
 while (true)
 { traiteSouris();
 traiteClavier();
 draw()
 actualiseAffichage()
 }
 }
 
 */

// ***********************************************************************
// cette fonction est appelée à chaque clic souris

void mousePressed() // on appuyé sur la touche
{
  if (sqrt(sq(mouseX-400)+sq(mouseY-400))<40) // on est au centre dans un rayon de 40 pixels
    envoiCaractere++;

  bt2.graphprint("On  [");
  bt2.graphprint(mouseX);
  bt2.graphprint(", ");
  bt2.graphprint(mouseY);
  bt2.graphprintln("]");
}

void mouseReleased() // on a relache la touche
{
  bt2.graphprint("Off [");
  bt2.graphprint(mouseX);
  bt2.graphprint(", ");
  bt2.graphprint(mouseY);
  bt2.graphprintln("]");
}
// ***********************************************************************

int compt=0; // compteur de delay dans le cas 1
int tempsFinPremierTravail=0;
void draw()
{
  println(etat);
  switch(etat) {
  case 0: 
    if (premierTravail())
    {
      etat=1; // si succes, on avance d'un cran
      tempsFinPremierTravail=millis();
    }
    break; // sortie du switch
  case 1 :
    if (deuxiemeTravail(tempsFinPremierTravail+4000)) etat=2; // si succes on change d'état
    break; 

  case 2: 
    troixiemeTravail();
    etat=3;
    break;
  case 3: 
    quatriemeTravail();
    // on est dans l'état ultime de fonctionnement
    break;
  }

  bt1.blinkCurseur(); // faire clignotter le curseur de la boite texte si le curseur à été demandé
}
